package com.example.project_musicalquiz.data.network

import android.util.Log
import com.example.project_musicalquiz.data.models.Album
import com.example.project_musicalquiz.data.models.Track
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.io.IOException

/**
 * [DeezerRemoteDataSource] is responsible for fetching data from the Deezer API.
 * It uses the [DeezerService] to make network requests and handles potential errors.
 *
 * @property deezerService The service used to interact with the Deezer API.
 * Defaults to an instance provided by [DeezerRetrofit].
 */
class DeezerRemoteDataSource(private val deezerService: DeezerService = DeezerRetrofit.retrofitDeezerService) {

    /**
     * Fetches a list of tracks from the Deezer API based on a search query.
     *
     * @param query The search query used to find tracks.
     * @return A list of [Track] objects matching the query, or an empty list if an error occurs.
     */
    suspend fun fetchTracks(query: String): List<Track> {
        return try {
            // Execute the network request on the IO dispatcher.
            withContext(Dispatchers.IO) {
                val response = deezerService.searchTracksByQuery(query)
                // Return the list of tracks from the response.
                response.tracks
            }
        } catch (e: IOException) {
            // Log the error and return an empty list.
            Log.e("DeezerRemoteDataSource", "Error fetching tracks", e)
            emptyList()
        }
    }

    /**
     * Fetches a list of albums from the Deezer API based on a search query.
     *
     * @param query The search query used to find albums.
     * @return A list of [Album] objects matching the query, or an empty list if an error occurs.
     */
    suspend fun fetchAlbums(query: String): List<Album> {
        return try {
            // Execute the network request on the IO dispatcher.
            withContext(Dispatchers.IO) {
                val response = deezerService.searchAlbumsByQuery(query)
                // Log the response for debugging purposes.
                Log.d("DeezerRemoteDataSource", "fetchAlbums: response.albums = ${response.albums}")
                // Return the list of albums from the response.
                response.albums
            }
        } catch (e: IOException) {
            // Log the error and return an empty list.
            Log.e("DeezerRemoteDataSource", "Error fetching albums", e)
            emptyList()
        }
    }
}