package com.example.project_musicalquiz.data.network

import com.example.project_musicalquiz.data.models.AlbumResponse
import com.example.project_musicalquiz.data.models.TrackResponse
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import retrofit2.http.GET
import retrofit2.http.Query

/**
 * The base URL for the Deezer API. All requests will be made relative to this URL.
 */
private const val DEEZER_API_BASE_URL = "https://api.deezer.com/"

/**
 * [DeezerService] defines the API endpoints for interacting with the Deezer API.
 * It uses Retrofit annotations to describe HTTP requests and their parameters.
 */
interface DeezerService {

    /**
     * Searches for tracks on the Deezer API based on a query.
     *
     * @param query The search query string.
     * @return A [TrackResponse] object containing the search results.
     */
    @GET("search/track")
    suspend fun searchTracksByQuery(@Query("q") query: String): TrackResponse

    /**
     * Searches for albums on the Deezer API based on a query.
     *
     * @param query The search query string.
     * @return An [AlbumResponse] object containing the search results.
     */
    @GET("search/album")
    suspend fun searchAlbumsByQuery(@Query("q") query: String): AlbumResponse
}

/**
 * [DeezerRetrofit] is a singleton object responsible for creating and providing
 * an instance of the [DeezerService] using Retrofit.
 */
object DeezerRetrofit {

    /**
     * The Retrofit instance used to create the [DeezerService].
     * It's configured with the base URL and a Gson converter.
     */
    private val retrofit = Retrofit.Builder()
        .addConverterFactory(GsonConverterFactory.create()) // Uses Gson for JSON conversion
        .baseUrl(DEEZER_API_BASE_URL) // Sets the base URL of the API
        .build()

    /**
     * Provides a lazily initialized instance of the [DeezerService].
     * This ensures that the service is created only when it's first needed.
     */
    val retrofitDeezerService: DeezerService by lazy {
        retrofit.create(DeezerService::class.java) // Creates the service from the interface
    }
}