package com.example.project_musicalquiz.search

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.example.project_musicalquiz.data.models.Album
import com.example.project_musicalquiz.data.models.Track
import com.example.project_musicalquiz.databinding.ItemAlbumBinding
import com.example.project_musicalquiz.databinding.ItemTrackBinding

/**
 * [SearchAdapter] is an adapter for displaying a list of search items (Tracks or Albums) in a RecyclerView.
 * It manages the display of items and user interactions (single click and long click).
 *
 * @param onItemClicked Lambda function called when an item is clicked.
 * @param onItemLongClicked Lambda function called when an item is long-clicked.
 */
class SearchAdapter(
    private val onItemClicked: (Any) -> Unit,
    private val onItemLongClicked: (Any) -> Unit
) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    /**
     * [items] is the list of items to display.
     * It can contain objects of type [Track] or [Album].
     */
    private var items: List<Any> = emptyList()

    /**
     * Submits a new list of items to be displayed.
     * This method efficiently updates the adapter's data and refreshes the RecyclerView using [DiffUtil].
     * It calculates the difference between the old and new lists and dispatches the minimum set of updates.
     *
     * @param newItems The new list of items to be displayed.
     */
    fun updateItems(newItems: List<Any>) {
        // Calculate the difference between the old and new lists using DiffUtil.
        val diffResult = DiffUtil.calculateDiff(SearchDiffCallback(items, newItems))
        // Update the items list with the new list.
        items = newItems
        // Dispatch the updates to the adapter, which will trigger the necessary changes in the RecyclerView.
        diffResult.dispatchUpdatesTo(this)
    }

    /**
     * [SearchDiffCallback] is a [DiffUtil.Callback] implementation used to calculate the difference
     * between two lists of search items. It determines which items have been inserted, removed, moved, or changed.
     *
     * @property oldList The old list of items.
     * @property newList The new list of items.
     */
    class SearchDiffCallback(
        private val oldList: List<Any>,
        private val newList: List<Any>
    ) : DiffUtil.Callback() {
        /**
         * Returns the size of the old list.
         *
         * @return The size of the old list.
         */
        override fun getOldListSize(): Int = oldList.size

        /**
         * Returns the size of the new list.
         *
         * @return The size of the new list.
         */
        override fun getNewListSize(): Int = newList.size

        /**
         * Determines if two items at the specified positions represent the same item.
         * This is used by DiffUtil to identify if an item has been moved or if it's a completely new item.
         *
         * @param oldItemPosition The position of the item in the old list.
         * @param newItemPosition The position of the item in the new list.
         * @return True if the items are the same (e.g., same ID), false otherwise.
         */
        override fun areItemsTheSame(oldItemPosition: Int, newItemPosition: Int): Boolean {
            // Get the items at the specified positions.
            val oldItem = oldList[oldItemPosition]
            val newItem = newList[newItemPosition]
            // Check if the items are the same based on their type and ID.
            return when {
                // If both items are Albums, compare their IDs.
                oldItem is Album && newItem is Album -> oldItem.id == newItem.id
                // If both items are Tracks, compare their IDs.
                oldItem is Track && newItem is Track -> oldItem.id == newItem.id
                // Otherwise, the items are not the same.
                else -> false
            }
        }

        /**
         * Determines if two items at the specified positions have the same data.
         * This is called only if [areItemsTheSame] returns true.
         * It checks if the contents of the items are the same (e.g., title, artist, cover).
         *
         * @param oldItemPosition The position of the item in the old list.
         * @param newItemPosition The position of the item in the new list.
         * @return True if the items have the same data, false otherwise.
         */
        override fun areContentsTheSame(oldItemPosition: Int, newItemPosition: Int): Boolean {
            // Get the items at the specified positions.
            val oldItem = oldList[oldItemPosition]
            val newItem = newList[newItemPosition]
            // Check if the items have the same content based on their type.
            return when {
                // If both items are Albums, compare their contents.
                oldItem is Album && newItem is Album -> oldItem == newItem
                // If both items are Tracks, compare their contents.
                oldItem is Track && newItem is Track -> oldItem == newItem
                // Otherwise, the items do not have the same content.
                else -> false
            }
        }
    }

    /**
     * Defines the view types for the RecyclerView.
     * [VIEW_TYPE_ALBUM] represents the view type for an Album item.
     * [VIEW_TYPE_TRACK] represents the view type for a Track item.
     */
    companion object {
        private const val VIEW_TYPE_ALBUM = 1
        private const val VIEW_TYPE_TRACK = 2
    }

    /**
     * Determines the view type for the item at the given position.
     *
     * @param position The position of the item in the list.
     * @return The view type of the item.
     * @throws IllegalArgumentException if the item type is unknown.
     */
    override fun getItemViewType(position: Int): Int {
        // Determine the view type based on the item type.
        return when (items[position]) {
            // If the item is an Album, return the Album view type.
            is Album -> VIEW_TYPE_ALBUM
            // If the item is a Track, return the Track view type.
            is Track -> VIEW_TYPE_TRACK
            // If the item type is unknown, throw an exception.
            else -> throw IllegalArgumentException("Unknown item type")
        }
    }

    /**
     * Creates a new ViewHolder based on the view type.
     *
     * @param parent The ViewGroup into which the new View will be added.
     * @param viewType The view type of the new View.
     * @return A new ViewHolder that holds a view to display an item.
     * @throws IllegalArgumentException if the view type is invalid.
     */
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        // Inflate the layout based on the view type.
        val inflater = LayoutInflater.from(parent.context)
        return when (viewType) {
            // If the view type is Album, inflate the Album layout and create an AlbumViewHolder.
            VIEW_TYPE_ALBUM -> {
                val binding = ItemAlbumBinding.inflate(inflater, parent, false)
                AlbumViewHolder(binding, onItemClicked, onItemLongClicked)
            }
            // If the view type is Track, inflate the Track layout and create a TrackViewHolder.
            VIEW_TYPE_TRACK -> {
                val binding = ItemTrackBinding.inflate(inflater, parent, false)
                TrackViewHolder(binding, onItemClicked, onItemLongClicked)
            }
            // If the view type is invalid, throw an exception.
            else -> throw IllegalArgumentException("Invalid view type")
        }
    }

    /**
     * Binds the data to a ViewHolder.
     *
     * @param holder The ViewHolder that should be updated.
     * @param position The position of the item in the list.
     */
    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        // Get the item at the current position.
        val item = items[position]
        // Bind the item to the ViewHolder based on its type.
        when (holder) {
            // If the holder is an AlbumViewHolder, bind the Album data.
            is AlbumViewHolder -> holder.bindItem(item as Album)
            // If the holder is a TrackViewHolder, bind the Track data.
            is TrackViewHolder -> holder.bindItem(item as Track)
        }
    }

    /**
     * Returns the total number of items in the list.
     *
     * @return The number of items in the list.
     */
    override fun getItemCount(): Int = items.size

    /**
     * [AlbumViewHolder] is the ViewHolder for displaying an album.
     *
     * @property binding The binding object for the album item layout.
     * @property onItemClicked Lambda function called when an album item is clicked.
     * @property onItemLongClicked Lambda function called when an album item is long-clicked.
     */
    inner class AlbumViewHolder(
        private val binding: ItemAlbumBinding,
        private val onItemClicked: (Any) -> Unit,
        private val onItemLongClicked: (Any) -> Unit
    ) : RecyclerView.ViewHolder(binding.root) {
        /**
         * Binds the album data to the view.
         *
         * @param item The [Album] item to bind.
         */
        fun bindItem(item: Album) {
            // Apply the data to the view using the binding object.
            binding.apply {
                // Set the album title.
                albumTitleTextView.text = item.title
                // Set the artist name.
                artistNameTextView.text = item.artist.name
                // Load the album cover image using Glide.
                Glide.with(itemView)
                    .load(item.cover_big)
                    .into(albumCoverImageView)
                // Set the click listener for the root view.
                root.setOnClickListener {
                    onItemClicked(item)
                }
                // Set the long click listener for the root view.
                root.setOnLongClickListener {
                    onItemLongClicked(item)
                    true
                }
            }
        }
    }

    /**
     * [TrackViewHolder] is the ViewHolder for displaying a track.
     *
     * @property binding The binding object for the track item layout.
     * @property onItemClicked Lambda function called when a track item is clicked.
     * @property onItemLongClicked Lambda function called when a track item is long-clicked.
     */
    inner class TrackViewHolder(
        private val binding: ItemTrackBinding,
        private val onItemClicked: (Any) -> Unit,
        private val onItemLongClicked: (Any) -> Unit
    ) : RecyclerView.ViewHolder(binding.root) {
        /**
         * Binds the track data to the view.
         *
         * @param item The [Track] item to bind.
         */
        fun bindItem(item: Track) {
            // Apply the data to the view using the binding object.
            binding.apply {
                // Set the track title.
                trackTitle.text = item.title
                // Set the artist name.
                trackArtist.text = item.artist.name
                // Load the track cover image using Glide.
                Glide.with(itemView)
                    .load(item.album.cover_medium)
                    .into(trackCoverImage)
                // Set the click listener for the root view.
                root.setOnClickListener {
                    onItemClicked(item)
                }
                // Set the long click listener for the root view.
                root.setOnLongClickListener {
                    onItemLongClicked(item)
                    true
                }
            }
        }
    }
}