package com.example.project_musicalquiz.search

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast

import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager

import com.example.project_musicalquiz.data.models.Album
import com.example.project_musicalquiz.data.models.Track
import com.example.project_musicalquiz.databinding.FragmentSearchBinding

/**
 * [SearchFragment] is a Fragment responsible for handling the search functionality
 * within the application. It allows users to search for tracks or albums and
 * displays the results in a RecyclerView.
 */
class SearchFragment : Fragment() {

    /**
     * Binding object for the fragment's layout.
     */
    private lateinit var binding: FragmentSearchBinding

    /**
     * ViewModel instance for managing search-related data and operations.
     */
    private val searchViewModel: SearchViewModel by viewModels()

    /**
     * Adapter for the RecyclerView to display search results.
     */
    private lateinit var searchAdapter: SearchAdapter

    /**
     * Called to have the fragment instantiate its user interface view.
     *
     * @param inflater The LayoutInflater object that can be used to inflate
     * any views in the fragment.
     * @param container If non-null, this is the parent view that the fragment's
     * UI should be attached to.
     * @param savedInstanceState If non-null, this fragment is being re-constructed
     * from a previous saved state as given here.
     * @return Return the View for the fragment's UI, or null.
     */
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        // Inflate the layout for this fragment using View Binding.
        binding = FragmentSearchBinding.inflate(inflater, container, false)
        return binding.root
    }

    /**
     * Called immediately after onCreateView() has returned, but before any saved state has been restored in to the view.
     *
     * @param view The View returned by onCreateView(LayoutInflater, ViewGroup, Bundle).
     * @param savedInstanceState If non-null, this fragment is being re-constructed
     * from a previous saved state as given here.
     */
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        // Initialize the adapter for the RecyclerView.
        initializeSearchAdapter()
        // Set up the RecyclerView with the adapter.
        setupRecyclerView()
        // Set up the search button's click listener.
        setupSearchButton()
        // Observe search results from the ViewModel.
        observeSearchResults()
        // Observe search errors from the ViewModel.
        observeSearchErrors()
    }

    /**
     * Initializes the [SearchAdapter] with click listeners for items.
     */
    private fun initializeSearchAdapter() {
        searchAdapter = SearchAdapter(
            onItemClicked = { item ->
                // Show a Toast message when an item is clicked.
                Toast.makeText(context, "Click: $item", Toast.LENGTH_SHORT).show()
            },
            onItemLongClicked = { item ->
                // Show a Toast message when an item is long-clicked.
                Toast.makeText(context, "Long Click: $item", Toast.LENGTH_SHORT).show()
            }
        )
    }

    /**
     * Observes the error LiveData from the [SearchViewModel] and displays a Toast
     * message when an error occurs.
     */
    private fun observeSearchErrors() {
        searchViewModel.searchError.observe(viewLifecycleOwner) { errorMessage ->
            // Display the error message in a Toast.
            Toast.makeText(context, errorMessage, Toast.LENGTH_LONG).show()
        }
    }

    /**
     * Sets up the RecyclerView with the [SearchAdapter].
     */
    private fun setupRecyclerView() {
        binding.recyclerView.apply {
            // Set the adapter for the RecyclerView.
            adapter = searchAdapter
        }
    }

    /**
     * Sets up the search button's click listener to initiate a search.
     */
    private fun setupSearchButton() {
        binding.searchButton.setOnClickListener {
            // Get the search query from the EditText.
            val query = binding.searchEditText.text.toString()
            if (query.isNotBlank()) {
                // Determine the search option (TRACK or ALBUM) based on the selected RadioButton.
                val searchOption =
                    if (binding.tracks.isChecked) SearchOption.TRACK else SearchOption.ALBUM
                // Initiate the search with the query and search option.
                initiateSearch(query, searchOption)
            } else {
                // Show a Toast message if the search query is empty.
                Toast.makeText(context, "Enter a search", Toast.LENGTH_SHORT).show()
            }
        }
    }

    /**
     * Initiates a search by calling the [SearchViewModel] to search for items.
     *
     * @param query The search query string.
     * @param searchOption The type of search (TRACK or ALBUM).
     */
    private fun initiateSearch(query: String, searchOption: SearchOption) {
        searchViewModel.performSearch(query, searchOption)
    }

    /**
     * Observes the search results LiveData from the [SearchViewModel] and updates
     * the [SearchAdapter] with the new results.
     */
    private fun observeSearchResults() {
        searchViewModel.searchResults.observe(viewLifecycleOwner) { searchResults ->
            // Update the adapter with the new search results.
            searchAdapter.updateItems(searchResults)
            // Update the layout manager based on the search results.
            updateLayoutManager(searchResults)
        }
    }

    /**
     * Updates the layout manager of the RecyclerView based on the type of search results.
     *
     * @param searchResults The list of search results.
     */
    private fun updateLayoutManager(searchResults: List<Any>) {
        val layoutManager = when {
            // If all results are Tracks, use a LinearLayoutManager.
            searchResults.all { it is Track } -> LinearLayoutManager(context)
            // If all results are Albums, use a GridLayoutManager with 2 columns.
            searchResults.all { it is Album } -> GridLayoutManager(context, 2)
            // If results are mixed (Tracks and Albums), use a GridLayoutManager with custom span sizes.
            else -> {
                // Define a custom SpanSizeLookup to handle different item types.
                val spanSizeLookup = object : GridLayoutManager.SpanSizeLookup() {
                    override fun getSpanSize(position: Int): Int {
                        // Tracks take 1 column, Albums take 2 columns.
                        return when (searchResults[position]) {
                            is Track -> 1
                            is Album -> 2
                            else -> 1
                        }
                    }
                }
                // Create a GridLayoutManager with 2 columns.
                val gridLayoutManager = GridLayoutManager(context, 2)
                // Set the custom SpanSizeLookup.
                gridLayoutManager.spanSizeLookup = spanSizeLookup
                gridLayoutManager
            }
        }
        // Set the layout manager for the RecyclerView.
        binding.recyclerView.layoutManager = layoutManager
    }
}