package com.example.project_musicalquiz.search

import android.app.Application
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.example.project_musicalquiz.data.models.Album
import com.example.project_musicalquiz.data.models.Track
import com.example.project_musicalquiz.data.network.DeezerRemoteDataSource
import kotlinx.coroutines.launch

/**
 * [SearchViewModel] is a ViewModel class responsible for managing and providing
 * data related to the search functionality within the application. It acts as
 * an intermediary between the UI (typically a Fragment or Activity) and the
 * data layer, specifically interacting with the [DeezerRemoteDataSource] to fetch
 * tracks and albums from the Deezer API.
 *
 * This ViewModel leverages coroutines for asynchronous operations and LiveData
 * to provide data updates to the UI in a lifecycle-aware manner.
 *
 * @param application The application instance, required for extending [AndroidViewModel].
 */
class SearchViewModel(application: Application) : AndroidViewModel(application) {

    /**
     * [deezerRemoteDataSource] is a private instance of [DeezerRemoteDataSource] used to communicate
     * with the Deezer API. It encapsulates the network logic for fetching data.
     */
    private val deezerRemoteDataSource = DeezerRemoteDataSource()

    /**
     * [_mutableSearchResults] is a private [MutableLiveData] that holds the list of search
     * results. It can contain a mixed list of [Track] or [Album] objects,
     * depending on the search query and type.
     *
     * This is an internal mutable version of the data, only modifiable within
     * the ViewModel.
     */
    private val _mutableSearchResults = MutableLiveData<List<Any>>()

    /**
     * [searchResults] is a public [LiveData] that exposes the list of search results
     * to the UI. Observers can subscribe to this LiveData to receive updates
     * whenever the search results change.
     *
     * This is the read-only version of [_mutableSearchResults], ensuring data integrity.
     */
    val searchResults: LiveData<List<Any>> = _mutableSearchResults

    /**
     * [_mutableSearchError] is a private [MutableLiveData] that holds an error message in case
     * of a failure during a network operation or data processing.
     *
     * This is an internal mutable version of the data, only modifiable within
     * the ViewModel.
     */
    private val _mutableSearchError = MutableLiveData<String>()

    /**
     * [searchError] is a public [LiveData] that exposes an error message to the UI.
     * Observers can subscribe to this LiveData to receive updates when an error
     * occurs, allowing the UI to display appropriate feedback to the user.
     *
     * This is the read-only version of [_mutableSearchError], ensuring data integrity.
     */
    val searchError: LiveData<String> = _mutableSearchError

    /**
     * [performSearch] initiates a search for items (tracks or albums) based on the
     * provided query and search type. It uses coroutines to perform the search
     * asynchronously and updates the [_mutableSearchResults] LiveData with the results.
     *
     * If an error occurs during the search, it updates the [_mutableSearchError] LiveData
     * with an appropriate error message.
     *
     * @param query The search query string.
     * @param searchOption The type of search to perform, either [SearchOption.TRACK]
     *                     or [SearchOption.ALBUM].
     */
    fun performSearch(query: String, searchOption: SearchOption) {
        viewModelScope.launch {
            try {
                // Perform the search based on the selected search option.
                val items = when (searchOption) {
                    // If the search option is TRACK, search for tracks.
                    SearchOption.TRACK -> deezerRemoteDataSource.fetchTracks(query)
                    // If the search option is ALBUM, search for albums.
                    SearchOption.ALBUM -> deezerRemoteDataSource.fetchAlbums(query)
                }
                // Update the search results LiveData with the fetched items.
                _mutableSearchResults.value = items
            } catch (e: Exception) {
                // If an error occurs, update the error LiveData with an error message.
                _mutableSearchError.value = "Error fetching items: ${e.message}"
            }
        }
    }
}